 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include "uphy_common.h"
#include "uphy_version.h"


namespace UPHY
{
    class DataSetKey
    {
        public:
            dataset_type_t  m_type;
            Version         m_version;

        private:
            DataSetKey()
                 : m_type(DataSetType_None), m_version(Version(0,0))
            {
            }

        public:
            DataSetKey(dataset_type_t type, const Version & version)
                 : m_type(type), m_version(version)
            {
            }

        public:
            DataSetKey(const DataSetKey & key)
                 : m_type(key.m_type), m_version(key.m_version)
            {
            }

        public:
            inline bool operator == (const DataSetKey & rhs ) const {
                return m_type == rhs.m_type && m_version == rhs.m_version;
            }

            inline bool operator != (const DataSetKey & rhs ) const {
                return m_type != rhs.m_type || m_version != rhs.m_version;
            }

            //
            // Warning !!! Used reverse sort. Inverted operators
            // For example 7.0 < 6.0 or 6.5 < 6.3
            // This need for correct work lower_bound in compatible search
            //
            inline bool operator > (const DataSetKey & rhs ) const {
                return m_type >= rhs.m_type ?
                        m_type == rhs.m_type && m_version < rhs.m_version : true;
            }

            inline bool operator < (const DataSetKey & rhs ) const {
                return m_type <= rhs.m_type ?
                        m_type == rhs.m_type && m_version > rhs.m_version : true;
            }
    };
};
